var observers = [];
var intervalMS = 1000;

var tick = function () {
  if (self.observers.length > 0) {
    self.observers.forEach(function (observer) { return observer(); });
    self.startTick();
  }
};

var startTick = function () {
  setTimeout(self.tick, self.intervalMS);
};

var unsubscribeTicker = function (observer) {
  var removeIndex = self.observers.indexOf(observer);
  if (removeIndex !== -1) {
    self.observers = self.observers.slice(removeIndex, 1);
  }
};

var subscribeTicker = function (observer) {
  var ref = self.observers;
  var length = ref.length;
  self.observers.push(observer);

  if (self.observers.length > 0 && length === 0) {
    self.startTick();
  }

  return function () { return self.unsubscribeTicker(observer); };
};

var changeInterval = function (newIntervalMS) {
  self.intervalMS = newIntervalMS;
};

var self = {
  observers: observers,
  intervalMS: intervalMS,
  tick: tick,
  startTick: startTick,
  unsubscribeTicker: unsubscribeTicker,
  subscribeTicker: subscribeTicker,
  changeInterval: changeInterval
};

function secondsToHuman(duration) {
  if ( duration === void 0 ) duration = 0;

  if (!duration) {
    return "00:00:00";
  }

  var seconds = parseInt(duration % 60, 10);
  var minutes = parseInt((duration / 60) % 60, 10);
  var hours = parseInt(Math.floor(duration / 3600), 10);

  hours = hours < 10 ? ("0" + hours) : hours;
  minutes = minutes < 10 ? ("0" + minutes) : minutes;
  seconds = seconds < 10 ? ("0" + seconds) : seconds;

  return (hours + ":" + minutes + ":" + seconds);
}

function secondsToMinutesAndSeconds(duration) {
  if ( duration === void 0 ) duration = 0;

  if (!duration) {
    return "00:00";
  }

  var seconds = parseInt(duration % 60, 10);
  var minutes = parseInt((duration / 60) % 60, 10);

  minutes = minutes < 10 ? ("0" + minutes) : minutes;
  seconds = seconds < 10 ? ("0" + seconds) : seconds;

  return (minutes + ":" + seconds);
}

//

var formats = {
  SECONDS: "SECONDS",
  MINUTES: "MINUTES",
  HOURS: "HOURS"
};
var formatters = {};
formatters[formats.SECONDS] = function (val) { return val; };
formatters[formats.MINUTES] = secondsToMinutesAndSeconds;
formatters[formats.HOURS] = secondsToHuman;

var script = {
  name: "VueTimeTicker",
  props: {
    value: {
      type: String,
      required: true
    },
    format: {
      type: String,
      validator: function (value) { return Object.values(formats).includes(value); },
      default: formats.MINUTES
    }
  },
  data: function data() {
    return {
      time: new Date(),
      unsibscribe: null
    };
  },
  computed: {
    formattedDate: function formattedDate() {
      var date = new Date(this.value);
      if (date.toDateString() === "Invalid Date") {
        return null;
      }

      var deltaSeconds = Math.abs(((this.time - date) / 1000).toFixed(0));
      return formatters[this.format](deltaSeconds);
    }
  },
  mounted: function mounted() {
    this.unsibscribe = subscribeTicker(this.repaint);
  },
  beforeDestroy: function beforeDestroy() {
    this.unsibscribe();
  },
  methods: {
    repaint: function repaint() {
      this.time = new Date();
    }
  }
};

function normalizeComponent(template, style, script, scopeId, isFunctionalTemplate, moduleIdentifier
/* server only */
, shadowMode, createInjector, createInjectorSSR, createInjectorShadow) {
  if (typeof shadowMode !== 'boolean') {
    createInjectorSSR = createInjector;
    createInjector = shadowMode;
    shadowMode = false;
  } // Vue.extend constructor export interop.


  var options = typeof script === 'function' ? script.options : script; // render functions

  if (template && template.render) {
    options.render = template.render;
    options.staticRenderFns = template.staticRenderFns;
    options._compiled = true; // functional template

    if (isFunctionalTemplate) {
      options.functional = true;
    }
  } // scopedId


  if (scopeId) {
    options._scopeId = scopeId;
  }

  var hook;

  if (moduleIdentifier) {
    // server build
    hook = function hook(context) {
      // 2.3 injection
      context = context || // cached call
      this.$vnode && this.$vnode.ssrContext || // stateful
      this.parent && this.parent.$vnode && this.parent.$vnode.ssrContext; // functional
      // 2.2 with runInNewContext: true

      if (!context && typeof __VUE_SSR_CONTEXT__ !== 'undefined') {
        context = __VUE_SSR_CONTEXT__;
      } // inject component styles


      if (style) {
        style.call(this, createInjectorSSR(context));
      } // register component module identifier for async chunk inference


      if (context && context._registeredComponents) {
        context._registeredComponents.add(moduleIdentifier);
      }
    }; // used by ssr in case component is cached and beforeCreate
    // never gets called


    options._ssrRegister = hook;
  } else if (style) {
    hook = shadowMode ? function () {
      style.call(this, createInjectorShadow(this.$root.$options.shadowRoot));
    } : function (context) {
      style.call(this, createInjector(context));
    };
  }

  if (hook) {
    if (options.functional) {
      // register for functional component in vue file
      var originalRender = options.render;

      options.render = function renderWithStyleInjection(h, context) {
        hook.call(context);
        return originalRender(h, context);
      };
    } else {
      // inject component registration as beforeCreate hook
      var existing = options.beforeCreate;
      options.beforeCreate = existing ? [].concat(existing, hook) : [hook];
    }
  }

  return script;
}

var normalizeComponent_1 = normalizeComponent;

/* script */
var __vue_script__ = script;

/* template */
var __vue_render__ = function () {var _vm=this;var _h=_vm.$createElement;var _c=_vm._self._c||_h;return _c('span',{staticClass:"time-counter"},[_vm._v(_vm._s(_vm.formattedDate))])};
var __vue_staticRenderFns__ = [];

  /* style */
  var __vue_inject_styles__ = undefined;
  /* scoped */
  var __vue_scope_id__ = undefined;
  /* module identifier */
  var __vue_module_identifier__ = undefined;
  /* functional template */
  var __vue_is_functional_template__ = false;
  /* style inject */
  
  /* style inject SSR */
  

  
  var component = normalizeComponent_1(
    { render: __vue_render__, staticRenderFns: __vue_staticRenderFns__ },
    __vue_inject_styles__,
    __vue_script__,
    __vue_scope_id__,
    __vue_is_functional_template__,
    __vue_module_identifier__,
    undefined,
    undefined
  );

function install(Vue) {
  if (install.installed) { return; }
  install.installed = true;
  Vue.component("VueTimeTicker", component);
}

var plugin = {
  install: install
};

var GlobalVue = null;
if (typeof window !== "undefined") {
  GlobalVue = window.Vue;
} else if (typeof global !== "undefined") {
  GlobalVue = global.vue;
}

if (GlobalVue) {
  GlobalVue.use(plugin);
}

component.install = install;

export default component;
